
import sys


import dropbox
import sys, os, json
import LwpMotoUtils
import urllib

global APP_KEY
global APP_SECRET
global TEMP_FOLDER

global DOCMOTO_BASE_FOLDER
global DOCMOTO_SERVER_URL
global DOCMOTO_USERNAME
global DOCMOTO_PWD

#In dropbox there is an folder Apps. Inside this will be created a folder with the name
#of this App. This is the folder that will be monitored for changes
#you can change this by adjusting the access_type from 'app_folder' to 'dropbox'
global ACCESS_TYPE


global STATE_FILE



def link(args):
    if len(args) != 1:
        sys.stderr.write("ERROR: \"link\" doesn't take any arguments.\n")
        sys.exit(1)

    sess = dropbox.session.DropboxSession(APP_KEY, APP_SECRET, ACCESS_TYPE)
    request_token = sess.obtain_request_token()

    # Make the user log in and authorize this token
    url = sess.build_authorize_url(request_token)
    sys.stdout.write("1. Go to: %s\n" % url)
    sys.stdout.write("2. Authorize this app.\n")
    sys.stdout.write("After you're done, press ENTER.\n")
    raw_input()

    # This will fail if the user didn't visit the above URL and hit 'Allow'
    access_token = sess.obtain_access_token(request_token)
    sys.stdout.write("Link successful.\n")

    save_state({
        'access_token': (access_token.key, access_token.secret)
    })
    
def update(count=None):

    page_limit = None
    

    # Load state
    state = load_state()
    access_token = state['access_token']
    cursor = state.get('cursor')

    # Connect to Dropbox
    try:
        sess = dropbox.session.DropboxSession(APP_KEY, APP_SECRET, ACCESS_TYPE)
        sess.set_token(*access_token)
        c = dropbox.client.DropboxClient(sess)
    except dropbox.rest.ErrorResponse, e:
        print "Update process error: " + str(e.status) + " " + str(e.reason) + "\n"
        return
    except:
        print "Unexpected update process error\n"
        return
    

    page = 0
    changed = False
    while (page_limit is None) or (page < page_limit):
        # Get /delta results from Dropbox
        try:
            result = c.delta(cursor)
        except dropbox.rest.ErrorResponse, e:
            print "Dropbox delta return error: " + str(e.status) + " " + str(e.reason) + "\n"
            return
        except:
            print "Unexpected update process Dropbox delta return error\n"
            return
    
        page += 1
        if result['reset'] == True:
            sys.stdout.write('reset\n')
            changed = True
        cursor = result['cursor']
        # Apply the entries one by one to DocMoto.
        for delta_entry in result['entries']:
            changed = True
            apply_delta(delta_entry, c)
        cursor = result['cursor']
        if not result['has_more']: break

    if not changed:
        sys.stdout.write("No updates " + str(count) + "\n")
    else:
        # Save state
        state['cursor'] = cursor
        save_state(state)
        

        
        
def apply_delta(entry, c):
    path, metadata = entry
    branch, leaf = split_path(path)
    root = DOCMOTO_SERVER_URL + "/" + DOCMOTO_BASE_FOLDER
    
   # Connect to Motie
    myRet = LwpMotoUtils.initialize(DOCMOTO_USERNAME,DOCMOTO_PWD,DOCMOTO_SERVER_URL)
    
    if not myRet['returnOK']:
        print "DocMoto connection error " + str(myRet['responseString']) + "\n"
        return
    
    if metadata is not None:
        sys.stdout.write('+ %s\n' % path)
        # Traverse down the tree until we find the parent folder of the entry
        # we want to add.  Create any missing folders along the way.
        myURL = root #Use str() to convert from unicode otherwise get problems with WebDAV
        myDropBoxPath = ""#DROPBOX_URL
        for part in branch:
            
            myURL = myURL + "/" + urllib.quote(str(part))
            myDropBoxPath = myDropBoxPath + "/" + part
            myRet = LwpMotoUtils.checkExists(myURL)
            if(not myRet['exists']):
                LwpMotoUtils.createCollection(myURL)
                
        # Create the file/folder.
        #node = get_or_create_child(children, leaf)
        #node.path = metadata['path']  # Save the un-lower-cased path.
        if metadata['is_dir']:
            # Only create an empty folder if there isn't one there already.
            myURL = myURL + "/" + urllib.quote(str(leaf))
            myRet = LwpMotoUtils.checkExists(myURL)
            if(not myRet['exists']):
                LwpMotoUtils.createCollection(myURL)
        
        else:
            #Go get the file, then put it into DocMoto
            myURL = myURL + "/" + urllib.quote(str(leaf))
            myDropBoxPath = myDropBoxPath + "/" + leaf
            writeFile(myDropBoxPath,leaf, c)
            sendFile(myURL,leaf)
            removeFile(leaf)
    
    else:
        sys.stdout.write('- %s\n' % path)
        # Traverse down the tree until we find the parent of the entry we
        # want to delete.
        myURL = root
        for part in branch:
            myURL = myURL + "/" + urllib.quote(str(part))
            myRet = LwpMotoUtils.checkExists(myURL)   
            if(not myRet['exists'] or not myRet['isCollection']): break
        else:
            # If we made it all the way, delete the file/folder (if it exists).
            # If this is a folder we need to add a final /.

            myURL = myURL + "/" + urllib.quote(str(leaf))
            myRet = LwpMotoUtils.checkExists(myURL)
            if (myRet['isCollection']):
                myURL = myURL + "/"
            
            #Delete it
            LwpMotoUtils.createDelete(myURL)


def split_path(path):
    assert path[0] == '/', path
    assert path != '/', path
    parts = path[1:].split('/')
    return parts[0:-1], parts[-1]



def reset(args):
    if len(args) != 1:
        sys.stderr.write("ERROR: \"reset\" takes no arguments.\n")
        sys.exit(1)

    # Delete cursor, empty tree.
    state = load_state()
    if 'cursor' in state:
        del state['cursor']
    save_state(state)

def load_state():
    if not os.path.exists(STATE_FILE):
        sys.stderr.write("ERROR: Couldn't find state file %r.  Run the \"link\" subcommand first.\n" % (STATE_FILE))
        sys.exit(1)
    f = open(STATE_FILE, 'r')
    state = json.load(f)
    f.close()
    return state

def save_state(state):
    f = open(STATE_FILE, 'w')
    json.dump(state, f, indent=4)
    f.close()

def sendFile(myURL,fileName):

    try:
        f = open(TEMP_FOLDER + fileName,'rb')
        buff = f.read()
        f.close()    
    except:
        print "Unexpected sendFile return error\n"
        return
    
    LwpMotoUtils.createPut(myURL,buff)
        
    # Add any properties. Correspond to DocMoto tags
    request = """<?xml version='1.0' encoding='utf-8'?>
              <propertyupdate xmlns='DAV:' xmlns:dm='http://www.docmoto.com/2008/03/uiconfig'>
                <set>
                  <prop>
                    <comment>Moved up by DropBox synchonizer</comment>                              
                  </prop>
                </set>
              </propertyupdate>"""                 
          
    # Confirm
    LwpMotoUtils.createPropPatch(myURL, request)
    
    LwpMotoUtils.createVersionControl(myURL)         

def writeFile(dropBoxPath,fileName, c):
    
    try:
        myFile = c.get_file(dropBoxPath).read()
    except dropbox.rest.ErrorResponse, e:
        print "Dropbox write file return error: " + str(e.error) + "\n"
        return
    except:
        print "Unexpected writeFile return error\n"
        return
        
    try:
        localFile = open(TEMP_FOLDER + fileName, 'w')
        localFile.write(myFile)
        localFile.close()
    except:
        print "Unexpected writeFile local error\n"
        return

def removeFile(fileName):
    
    try:
        os.remove(TEMP_FOLDER + fileName)
    except:
        print "Unexpected removeFile local\n"
        return        